package ucot;

import java.io.IOException;
import java.util.Properties;

import ucot.exceptions.BadPropertyValueException;

/**
 * <pre>public abstract interface <b>ModulePropertyInterface</b></pre>
 * 
 * <p>This interface defines required methods for another interfaces to support
 * properties in a way that the module implementing this interface would be
 * compatible with UCOT core.</p>
 * 
 * @see #ucot.ModuleProperties
 * @author tujupien
 */
public abstract interface ModulePropertyInterface {
	
	/**
	 * <p>Returns module's properties.</p>
	 * 
	 * @return Module's properties.
	 */
	public Properties getProperties();
	
	/**
	 * <p>Sets options for the adapter.</p>
	 * 
	 * <p>Notice that this does not need to be an perfect set
	 * of properties for this module because these properties
	 * should be merged to the current properties. So it is
	 * possible to change only one property value by giving a
	 * new property object with the new value for the given key.</p>
	 * 
	 * @param properties Properties for the adapter.
	 */
	public void setProperties(Properties properties);

	/**
	 * <p>Applies current properties for the module.</p>
	 * 
	 * @throws BadPropertyValueException In this case exception
	 *   is thrown only if either the given parser or heuristic
	 *   does not exist.
	 */
	public void applyProperties() throws BadPropertyValueException;
	
	/**
	 * <p>Saves current properties to the properties XML file.</p>
	 * 
	 * @throws IOException Exception is thrown if something went wrong.
	 */
	public void saveProperties() throws IOException;
	
	/**
	 * <p>Loads settings from the current properties XML file.</p>
	 * 
	 * @throws IOException Exception is thrown if something went wrong.
	 */
	public void loadProperties() throws IOException;
	
	/**
	 * <p>Method which returns the factory default properties
	 * for the module.</p>
	 * 
	 * @return Default properties.
	 */
	public Properties loadDefaultProperties();
	
}
