package ucot.model;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Set;


/**
 * <pre>public interface <b>ModelEditor</b></pre>
 * <p><tt>ModelEditor</tt> is a interface to edit analyze models.</p>
 * <p>Edition is made trough <tt>ModelEditor</tt> enable modifications
 * tracking and to protect the internal workings of the concrete implementation
 * of the analyze model.</p>
 * <p>
 * Every change to the <tt>AnalyzeModel</tt> should notify the
 * observers of the <tt>AnalyzeModel</tt> with appropriate <tt>Updation</tt>
 * object. See <tt>Updation</tt> object for additional information about
 * how to inform changes.
 * </p>
 * @author pajumasu
 * @see Updation for structure of information passed about modifications
 * @see AnalyzeModel for detailed explanation about the items found and editable in the model. 
 * 
 */
public interface ModelEditor {
	/**
	 * <p>Clears model and makes it empty.</p>
	 * <p>If cleared MUST send Updation message:<br/>
	 * Type: <tt>UpdationType.CLEAR</tt></p>
	 * 
	 * @see Updation.UpdationType#CLEAR
	 */
	public void clearModel();
	
	/**
	 * <p>Returns the names of the entities that the model contains.</p>
	 * 
	 * @return the names of the entities that the model contains.
	 */
	public Set<String> getEntityNames();
	
	/**
	 * <p>Adds an entity to the model. If entity exists nothing is done.</p> 	 
	 * <p>If addition occurs MUST send Updation message:<br/>
	 * Type: <tt>UpdationType.ADD</tt><br/>
	 * DataType: <tt>UpdationDataType.ENTITY</tt><br/>
	 * Parameters: name of the entity.</p>
	 * 
	 *
	 * @param name The name of the entity.
	 * @see Updation.UpdationType#ADD
	 * @see Updation.UpdationDataType#ENTITY
	 */
	public void addEntity(String name);
	
	/**
	 * <p>Removes entity from model. If no such entity is found
	 * nothing is done.</p>
	 * <p>If removing occurs MUST send Updation message:<br/>
	 * Type: <tt>UpdationType.DELETE</tt><br/>
	 * DataType: <tt>UpdationDataType.ENTITY</tt><br/>
	 * Parameters: name of the entity.</p>
	 * 
	 * @param name The name of the entity.
	 * @see Updation.UpdationType#ADD
	 * @see Updation.UpdationDataType#ENTITY
	 * @throws NoSuchEntityException If given entity is not found.
	 */
	public void removeEntity(String name);
	
	/**
	 * <p>Returns the type of the entity.</p>
	 * 
	 * @param entity The name of the entity.
	 * @return The type of the entity.
	 * @throws NoSuchEntityException If given entity is not found.
	 */
	public String getEntityType(String entity);
		
	/**
	 * <p>Sets the type of the entity.</p>
	 * <p>If type is set MUST send Updation message:<br/>
	 * Type: <tt>UpdationType.MODIFY</tt><br/>
	 * DataType: <tt>UpdationDataType.ENTITY_TYPE</tt><br/>
	 * Parameters: name of the entity, the type string</p>
	 * 
	 * @param entity The name of the entity.
	 * @param entityType The type
	 * @see Updation.UpdationType#MODIFY
	 * @see Updation.UpdationDataType#ENTITY_TYPE
	 * @throws NoSuchEntityException If given entity is not found.
	 */ 
	public void setEntityType(String entity, String entityType);
	
	/**
	 * <p>Changes the name of the entity.</p>
	 * <p>If change occurs MUST send Updation message:<br/>
	 * Type: <tt>UpdationType.MODIFY</tt><br/>
	 * DataType: <tt>UpdationDataType.ENTITY</tt><br/>
	 * Parameters: old name of the entity, new name of the entity</p>
	 * 
	 * @param oldName The entity name that is going to be changed.
	 * @param newName The new name for that entity.
	 * @see Updation.UpdationType#MODIFY
	 * @see Updation.UpdationDataType#ENTITY
	 * @throws NoSuchEntityException If given entity is not found.
	 */
	public void changeEntityName(String oldName, String newName) throws NoSuchEntityException;
	
	/**
	 * <p>Does the model contain the entity?</p>
	 * 
	 * @param entityName The name of the entity which existence is checked.
	 * @return true if the entity exists. False otherwise.
	 */
	public boolean containsEntity(String entityName);
	
	
	/**
	 * <p>Adds parent to the entity. If entity is not found </p>
	 * <p>if adding occurs MUST send Updation message:<br/>
	 * Type: <tt>UpdationType.ADD</tt><br/>
	 * DataType: <tt>UpdationDataType.ENTITY_PARENT</tt><br/>
	 * Parameters: name of the entity, the name of the parent</p>
	 * 
	 * @param entityName The name of the child entity.
	 * @param parentEntity The name of the parent entity.
	 * @throws NoSuchEntityException If given entity is not found.
	 * @see Updation.UpdationType#ADD
	 * @see Updation.UpdationDataType#ENTITY_PARENT
	 */
	public void addParent(String entityName, String parentEntity) throws NoSuchEntityException;
	
	/**
	 * <p>Removes given parent from the entity.</p>
	 * <p>If removing occurs MUST send Updation message:<br/>
	 * Type: <tt>UpdationType.DELETE</tt><br/>
	 * DataType: <tt>UpdationDataType.ENTITY_PARENT</tt><br/>
	 * Parameters: name of the entity, the name of the parent</p>
	 * 
	 * @param entityName The name of the child entity.
	 * @param parentEntity The name of the parent entity.
	 * @throws NoSuchEntityException If given entity is not found.
	 *  
	 * @see Updation.UpdationType#DELETE
	 * @see Updation.UpdationDataType#ENTITY_PARENT
	 */
	public void removeParent(String entityName, String parentEntity) throws AnalyzeModelException;
	
	/**
	 * <p>Removes all the parents of the entity.
	 * This does same thing than calling removeParent(String,String)
	 * for every parent.</p> 
	 * 
	 * @param entityName The name of the entity which parents are cleared.
	 * @throws NoSuchEntityException If given entity is not found.
	 */
	public void removeAllParents(String entityName) throws NoSuchEntityException;
	
	/**
	 * <p>Returns the parents of the entity.</p>
	 * @param entityName The name of the entity.
	 * @return The parents of the given entity.
	 * @throws NoSuchEntityException If given entity is not found.
	 */
	public Set<String> getParents(String entityName);
	
	/**
	 * <p>Adds child to the entity.
	 * This is same than calling addParent(childEntity, entityName)
	 * and same kind of updation message is expected.
	 * </p>
	 * <p>if adding occurs MUST send Updation message:<br/>
	 * Type: <tt>UpdationType.ADD</tt><br/>
	 * DataType: <tt>UpdationDataType.ENTITY_PARENT</tt><br/>
	 * Parameters: name of the entity, the name of the parent</p>
	 * 
	 * @param entityName The name of parent entity.
	 * @param childEntity The name of the child.
	 * @throws NoSuchEntityException If given entity is not found.
	 * 
	 * @see Updation.UpdationType#ADD
	 * @see Updation.UpdationDataType#ENTITY_PARENT
	 */
	public void addChild(String entityName, String childEntity) throws NoSuchEntityException;
	
	/**
	 * <p>Removes given child from the entity.
	 * This is same than calling removeParent(childEntity, entityName)
	 * and same kind of updation message is expected.</p>
	 * <p>If removing occurs MUST send Updation message:<br/>
	 * Type: <tt>UpdationType.DELETE</tt><br/>
	 * DataType: <tt>UpdationDataType.ENTITY_PARENT</tt><br/>
	 * Parameters: name of the entity, the name of the parent</p>
	 * 
	 * @param entityName The entity.
	 * @param childEntity The child to be removed.
	 * @throws NoSuchEntityException If given entity is not found.
	 * 
	 * @see Updation.UpdationType#DELETE
	 * @see Updation.UpdationDataType#ENTITY_PARENT
	 */
	public void removeChild(String entityName, String childEntity) throws AnalyzeModelException;
	
	/**
	 * <p>Removes all the children from the entity.
	 * This is same than calling removeChild(String, String)
	 * for all the childs.
	 * </p>
	 * @param entityName The name of the entity which childs are cleared.
	 * @throws NoSuchEntityException If given entity is not found.
	 */	
	public void removeAllChildren(String entityName) throws AnalyzeModelException;
	
	/**
	 * <p>Returns the names of the entity's children.</p>
	 * @param entityName The name of the entity.
	 * @return The child of the given entity.
	 * @throws NoSuchEntityException If given entity is not found.
	 */
	public Set<String> getChildren(String entityName);
	
	/**
	 * <p>Add method to the entity.</p>
	 * <p>If adding occurs MUST send Updation message:<br/>
	 * Type: <tt>UpdationType.ADD</tt><br/>
	 * DataType: <tt>UpdationDataType.METHOD</tt><br/>
	 * Parameters: name of the entity, the name of the method</p>
	 * 
	 * @param entityName The name of the entity.
	 * @param methodName The name of the method.
     * @see Updation.UpdationType#ADD
	 * @see Updation.UpdationDataType#METHOD 
	 * @throws NoSuchEntityException If given entity is not found.
	 */
	public void addMethod(String entityName, String methodName) throws NoSuchEntityException;
	
	/**
	 * <p>Removes method from the entity.<p>
	 * <p>If remove occurs MUST send Updation message:<br/>
	 * Type: <tt>UpdationType.REMOVE</tt><br/>
	 * DataType: <tt>UpdationDataType.METHOD</tt><br/>
	 * Parameters: name of the entity, the name of the method</p>
	 * 
	 * @param entityName The entity which owns the method.
	 * @param methodName The name of the method.
	 * @see Updation.UpdationType#REMOVE
	 * @see Updation.UpdationDataType#METHOD
	 * @throws NoSuchEntityException If given entity is not found.
	 */
	public void removeMethod(String entityName, String methodName) throws NoSuchEntityException;
	
	/**
	 * <p>Changes method name.</p>
	 * <p>If change occurs MUST send Updation message:<br/>
	 * Type: <tt>UpdationType.MODIFY</tt><br/>
	 * DataType: <tt>UpdationDataType.METHOD</tt><br/>
	 * Parameters: name of the entity, the old name of the method, the new name of the method.</p>
	 * 
	 * @param entityName The name of the entity.
	 * @param methodOldName
	 * @param methodNewName
	 * @see Updation.UpdationType#MODIFY
	 * @see Updation.UpdationDataType#METHOD
	 * @throws NoSuchEntityException If given entity is not found.
	 * @throws NoSuchMethodException If given method is not found.
	 */
	public void changeMethodName(String entityName, String methodOldName, String methodNewName) throws AnalyzeModelException;
	
	/**
	 * <p>Checks if the given method exists in the entity.</p>
	 * @param entityName The name of the entity
	 * @param methodName The name of the method.
	 * @return True if the entity has given method.
	 * @throws NoSuchEntityException If given entity is not found.
	 */
	public boolean containsMethod(String entityName, String methodName);
	
	/**
	 * <p>Returns the names of the entity's methods.</p>
	 * @param entityName The name of the entity
	 * @return The entity's methods' names.
	 * @throws NoSuchEntityException If given entity is not found.
	 */
	public Set<String> getMethodNames(String entityName);
	
	/**
	 * <p>Returns set of entities that are refered by methods
	 * in the model.</p>
	 *  
	 * @param entityName The name of the entity that owns the method.
	 * @param methodName The methods name.
	 * @return Set of entity names that are refered by the method in some way.
	 * @throws NoSuchEntityException If given entity is not found.
	 * @throws NoSuchMethodException If given method is not found.
	 */
	public Set<String> getEntitiesInfluencedByMethod(String entityName, String methodName);
	
	/**
	 * <p>Adds an influence between entity's method and the given entity.</p>
	 * <p>If addition occurs MUST send Updation message:<br/>
	 * Type: <tt>UpdationType.ADD</tt><br/>
	 * DataType: <tt>UpdationDataType.METHOD_INFLUENCE</tt><br/>
	 * Parameters: name of the entity, the name of the method, the name of entity influenced.</p>
	 * 
	 * @param entityName The name of the entity that owns the method.
	 * @param methodName The methods name.
	 * @param influencedEntity
	 * @throws NoSuchEntityException If given entity is not found.
	 * @throws NoSuchMethodException If given method is not found.
	 * @see Updation.UpdationType#ADD
	 * @see Updation.UpdationDataType#METHOD_INFLUENCE 
	 */
	public void addEntityInfluenceByMethod(String entityName, String methodName, String influencedEntity) throws AnalyzeModelException;
	
	
	/**
	 * <p>Removes influnce between entity's method and the given entity.</p>
	 * <p>If deletion occurs MUST send Updation message:<br/>
	 * Type: <tt>UpdationType.DELETE</tt><br/>
	 * DataType: <tt>UpdationDataType.METHOD_INFLUENCE</tt><br/>
	 * Parameters: name of the entity, the name of the method, the name of entity influenced.</p>
	 *	 
	 * @param entityName The name of the entity that owns the method.
	 * @param methodName The methods name.
	 * @param influencedEntity The name of the method that is influenced by the given method.
	 * @throws NoSuchEntityException If given entity is not found.
	 * @throws NoSuchMethodException If given method is not found. 
	 * @see Updation.UpdationType#DELETE
	 * @see Updation.UpdationDataType#METHOD_INFLUENCE 
	 */
	public void removeEntityInfluenceByMethod(String entityName, String methodName, String influencedEntity) throws AnalyzeModelException;
	
	
	/**
	 *<p>Adds attribute to the entity.</p>
	 *<p>If addition occurs MUST send Updation message:<br/>
	 * Type: <tt>UpdationType.ADD</tt><br/>
	 * DataType: <tt>UpdationDataType.ATTRIBUTE</tt><br/>
	 * Parameters: name of the entity, the name of the attribute</p>
	 * 
	 * @param entityName The name of the entity.
	 * @param attributeName The name of the attribute.
	 * @throws NoSuchEntityException If given entity is not found.
	 * @see Updation.UpdationType#ADD
	 * @see Updation.UpdationDataType#ATTRIBUTE 
	 */	
	public void addAttribute(String entityName, String attributeName) throws NoSuchEntityException;
	
	/**
	 * <p>Removes attribute from the entity.</p>
	 * <p>If deletion occurs MUST send Updation message:<br/>
	 * Type: <tt>UpdationType.DELETE</tt><br/>
	 * DataType: <tt>UpdationDataType.ATTRIBUTE</tt><br/>
	 * Parameters: name of the entity, the name of the attribute</p>
	 * 
	 * @param entityName The name of the entity.
	 * @param attributeName The name of the attribute.
	 * @throws NoSuchEntityException If given entity is not found.
	 * @throws NoSuchAttributeException If given attribute is not found.
	 * @see Updation.UpdationType#DELETE
	 * @see Updation.UpdationDataType#ATTRIBUTE 
	 */		
	public void removeAttribute(String entityName, String attributeName) throws AnalyzeModelException;
	
	/**
	 * <p>Checks wheter the entity contains the attribute or not.</p>
	 * @param entityName The name of the entity.
	 * @param attributeName The name of the attribute.
	 * @return True if contains.
	 * @throws NoSuchEntityException If given entity is not found.
	 */
	public boolean containsAttribute(String entityName, String attributeName) throws NoSuchEntityException;
	
	/**
	 * <p>Gets from part of the cardinality of the attribute relation.
	 * From part means the cardinality on the entity's side that owns the
	 * attribute. For example in one-to-many relation the 'one' is <i>from
	 * cardinality</i>.</p>
	 * 
	 * 
	 * @param entityName The name of the entity.
	 * @param attributeName The name of the attibute.
	 * @return The cardinality value of the from part of the cardinal relation.
	 */
	public String getAttributeFromCardinal(String entityName, String attributeName) throws AnalyzeModelException;
	
	
	/**
	 * <p>Gets to part of the cardinality of the attribute relation.
	 * To part means the cardinality on the attributes side. For example in
	 * one-to-many relation the 'many' is <i>to cardinality</i>.</p>
	 * 
	 * @param entityName The name of the entity.
	 * @param attributeName The name of the attibute.
	 * @return The cardinality value of the to part of the cardinal relation.
	 */
	public String getAttributeToCardinal(String entityName, String attributeName) throws AnalyzeModelException;
	
	/**
	 * <p>Sets the cardinality on the entitys side.</p> 
	 * <p>If change occurs MUST send Updation message:<br/>
	 * Type: <tt>UpdationType.MODIFY</tt><br/>
	 * DataType: <tt>UpdationDataType.ATTRIBUTE_FROM_CARDINALITY</tt><br/>
	 * Parameters: name of the entity, the name of the attribute, the from cardinality</p>
	 * 
	 * @param entityName The name of the entity.
	 * @param attributeName The name of the attibute.
	 * @param cardinal The cardinality value.
	 * @throws NoSuchEntityException If given entity is not found.
	 * @throws NoSuchAttributeException If given attribute is not found.
	 * @see Updation.UpdationType#MODIFY
	 * @see Updation.UpdationDataType#ATTRIBUTE_FROM_CARDINALITY 
	 * 
	 */
	public void setAttributeFromCardinal(String entityName, String attributeName, String cardinal) throws AnalyzeModelException;
	
	/**
	 * <p>Sets the cardinality on attributes side.</p>
	 * <p>If change occurs MUST send Updation message:<br/>
	 * Type: <tt>UpdationType.MODIFY</tt><br/>
	 * DataType: <tt>UpdationDataType.ATTRIBUTE_TO_CARDINALITY</tt><br/>
	 * Parameters: name of the entity, the name of the attribute, the to cardinality</p>
	 * 
	 * @param entityName The name of the entity.
	 * @param attributeName The name of the attibute.
	 * @param cardinal The cardinality value.
	 * @throws NoSuchEntityException If given entity is not found.
	 * @throws NoSuchAttributeException If given attribute is not found.
	 * @see Updation.UpdationType#MODIFY
	 * @see Updation.UpdationDataType#ATTRIBUTE_TO_CARDINALITY 
	 */
	public void setAttributeToCardinal(String entityName, String attributeName, String cardinal) throws AnalyzeModelException;
	
	/**
	 * <p>Returns the attributes of the entity.</p> 
	 * @param entityName The name of the entity.
	 * @return The attributes.
	 * @throws NoSuchEntityException If given entity is not found.
	 */
	public Set<String> getAttributeNames(String entityName);
	
	/**
	 * <p>Merges given <tt>AnalyzeModel</tt> to the editors model.
	 * Merge only adds things that do not yet exist in the current model.
	 * Nothing is delted or modified. For example attribute's cardinalities
	 * are not modified even if there is same attribute with different
	 * cardinalities.</p>
	 * <p>
	 * All proper updation messages should be sended for every action
	 * made. It might be good idea to send AnalydeModel.signalModificationStarted()
	 * when starting modifications and AnalzydeMode.signalReady() when done.
	 * </p>
	 * @param model The analyze model beign merged.
	 */
	public void merge(AnalyzeModel model) throws AnalyzeModelException;
	
	/**
	 * <p>Get updations done to this analyzemodel.</p> 
	 * @return List of Updations or null if this feature is not supported.
	 */
	public List<Updation> getUpdations();
	
	
	/**
	 * <p>Merges entities to other entity.	 
	 * If given target entity (called <tt>targetEntityName</tt>) does not exists
	 * it is created.</p>
	 * <p>
	 * All the methods, attributes, and parents are added to one
	 * entity and the sources are removed afterwards. If there
	 * are same attributes it depens on the underlaying 
	 * implementation which one of them will remain in
	 * the final entity.
	 * </p>
	 * 
	 * @param targetEntityName The name of the entity after merge.
	 * @param mergeSet The set of entity names that are going to be merged.
	 */
	public void mergeEntity(String targetEntityName, Set<String> mergeSet);
	
	/**
	 * <p>Undoes some edition steps.</p>
	 * @param steps to undo
	 */
	public void stepBack(int steps);
	
	/**
	 * <p>Executes action defined by updaton object.</p>
	 * @return true if success, false if did not.
	 */
	public boolean execute(Updation updation) throws AnalyzeModelException;
	
	
	
	/**
	 * <p>Signals all observers that this model is ready.</p>
	 * <p>Sends observers <tt>Updation<tt>-message which type
	 * is <tt>Updation.UpdationType.READY</tt>.
	 * @see #updationStartedSignal()
	 */	
	public void readySignal();
	
	/**
	 * <p>Signals all observers that this model is beign modified.</p>
	 * <p>
	 * This method should be called before the model is going to 
	 * trough lots of changes. All observers receive updation signal
	 * that is from this model and the argument is instance of <tt>Updation</tt>
	 * which type is <tt>Updation.UpdationType.MODIFICATION_STARTED</tt>.
	 * </p>
	 * <p>
	 * Example of Observer listening the model:
	 * </p>
	 * <pre>
	 * new Observable(){
	 * 		void update(Observable o, Object arg){
	 * 			// Check that we know how to handle the parameters.
	 * 			if (! (o instanceof AnalyzeModel)) return;
	 * 			if (! (arg instanceof Updation)) return;
	 * 
	 * 			Updation updation = (Updation) arg;
	 * 
	 * 			// Check updation type
	 * 			switch(updation.getType()){
	 * 				case Update.MODIFICATION_STARTED: 					
	 * 					drawUpdates = false;	// Dont draw updates.
	 * 				break;
 	 * 				case Update.READY:
 	 * 
 	 * 					drawUpdates = true;		// Start drawing updates.
	 * 				break;
	 * 			}
	 * 			// Draw updates if we are not in midle of updation.	  
	 * 			if (drawUpdates){
	 * 				doDrawUpdates();
	 * 			}
	 * 		}
	 * }
	 * </pre>
	 */
	public void updationStartedSignal();
	
	/**
	 * <p>Saves the updations (modification log) to the given file.</p>
	 * @param target Defines the target filename for the modification
	 *   log file.
	 */
	public void saveUpdationsToFile(File target) throws IOException;
	
}
