package ucot.ui.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashSet;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Properties;
import java.util.Set;
import java.util.Vector;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSplitPane;

import ucot.ModulePropertyInterface;
import ucot.core.ControlInterface;
import ucot.exceptions.BadPropertyValueException;
import ucot.input.UseCaseCollection;
import ucot.output.OutputInterface;
import ucot.ui.ProgressBarInterface;
import ucot.ui.UIInterface;
import ucot.ui.UseCasePanelInterface;
import ucot.ui.gui.dialog.AboutDialog;
import ucot.ui.gui.dialog.EntityPropertiesDialog;
import ucot.ui.gui.dialog.ProgressBarDialog;
import ucot.ui.gui.dialog.SettingsDialog;
import ucot.ui.gui.dot.DotColorTheme;
import ucot.ui.gui.dot.DotPanel;
import ucot.ui.gui.dot.DotPanel.ExportImageType;
import ucot.ui.gui.tree.analyzemodeltree.AnalyzeModelTree;
import ucot.ui.gui.tree.analyzemodeltree.AnalyzeTreeModel;
import ucot.ui.gui.tree.usecasetree.UseCaseTree;
import ucot.utils.CustomFileFilter;
import ucot.utils.FileTools;
import ucot.utils.PropertiesTools;
import ucot.utils.StringTools;

// FIXME warnings

/**
 * Simple graphical user interface for UCOT core which uses dot (in DotPanel) to
 * draw graphics. This class is inherited from JFrame and implements
 * the UIInterface defined in UCOT package.
 * 
 * @author ilanliuk, tujupien.
 */
public class GraphicalUI extends JFrame implements UIInterface, Observer, ModulePropertyInterface {
	
	private static final long serialVersionUID = -4246720431258449398L;
	
	private ControlInterface core = null;
	private ActionListener menuListener;
	private WindowListener windowListener;
	private AnalyzeModelTree analyzeModelTree;
	private JScrollPane modelTreeScrollPane;
	private UseCaseTree useCaseTree;
	private JScrollPane useCaseTreeScrollPane;
	private JMenuBar menu;
	private DotPanel dotPanel;
	private JScrollPane dotScrollPane;
	private JLabel dotTimer = new JLabel(" 0 ms ");
	private UseCasePanelInterface useCasePanel = 
		new SimpleUseCasePanel();
	private JScrollPane useCasePanelScrollPane = 
		new JScrollPane((JPanel)useCasePanel);
	private Statusbar statusbar = new Statusbar();
	private EntityPropertiesDialog entityPropertiesDialog = 
		new EntityPropertiesDialog(this);
	private ProgressBarDialog progressBarDialog;
	private URL currentFile;
	private boolean changesMade = false;
	private JLabel changesIndicator = new JLabel(" " + UNMODIFIED_STRING + " ");
	
	protected URL propertiesURL = getPropertiesURL();
	protected Properties properties = loadDefaultProperties();
	
	public List<String> entityTypes;
    
    protected static String SERIALIZATION_DESCRIPTION = "Java serialization file";
    protected static String[] SERIALIZATION_EXTENSIONS = {"ser"};
    
    protected static String PROGRAM_TITLE = "UCOT";
    
    protected final static String DEFAULT_ERROR_HEADER = "Error";
    protected final static String DEFAULT_QUESTION_HEADER =	"Question";
    protected final static String DEFAULT_WARNING_HEADER = "Warning";
    
    protected final static String EXPORT_ALL_GRAPH_FILENAME = "graph";
    protected final static String EXPORT_ALL_DATA_FILENAME = "data";
    protected final static String EXPORT_ALL_SERIALIZATION_FILENAME = "analyze_model";
    protected final static String EXPORT_ALL_LOG_FILENAME = "modification_log";
    
    // Property keys.
    public final static String PROPERTY_DOT_PATH = "DOT_PATH";
    public final static String PROPERTY_EPS_TO_PDF_PATH = "EPS_TO_PDF_PATH";
    public final static String PROPERTY_TEMP_INPUT_FILE = "TEMP_INPUT_FILE";
    public final static String PROPERTY_TEMP_OUTPUT_FILE = "TEMP_OUTPUT_FILE";
    public final static String PROPERTY_TEMP_EPS_FILE = "TEMP_EPS_FILE";
    public final static String PROPERTY_DOT_HIGHLIGHT_COLOR = "DOT_HIGHLIGHT_COLOR";
    public final static String PROPERTY_DOT_COLOR = "DOT_COLOR";
    public final static String PROPERTY_ENTITY_TYPES = "ENTITY_TYPES";
    public final static String PROPERTY_GRAPH_FONT_SIZE = "GRAPH_FONT_SIZE";
    
    // TODO localize us
    protected static String CLEAR_MODEL_QUESTION_HEADER = 
    	"Clear model";
    protected static String CLEAR_MODEL_QUESTION =
    	"Are you sure you want to clear\nthe entire analyze model?";
    protected static String NEW_MODEL_QUESTION_HEADER = 
    	"Create a new model";
    protected static String NEW_MODEL_QUESTION =
    	"Would you like to save changes to the current model\nbefore creating a new one?";
    protected static String QUIT_QUESTION_HEADER =
    	"Quit";
    protected static String QUIT_QUESTION =
    	"Are you sure you want to quit\nand lose all unsaved changes?";
    protected static String LOG_PREFIX = 
    	"GUI: ";
    protected static String LOG_FILE_LOAD_TRIGGERED = 
    	"User wants to load a file.";
    protected static String LOG_QUIT_PROGRAM_TRIGGERED = 
    	"User wants to quit program.";
    protected static String LOG_QUIT_PROGRAM = 
    	"Shutting down...";
    protected static String LOG_CLEAR_MODEL_TRIGGERED = 
    	"User wants to clear the entire analyze model.";
    protected static String LOG_SETTINGS_TRIGGERED =
    	"User wants to change program settings.";
    protected static String LOG_EXPORT_AS_IMAGE_TRIGGERED =
    	"Exporting analyze model as image.";
    protected static String LOG_EXPORT_DONE_MESSAGE =
    	"Analyze model successfully exported.";
    protected static String LOG_ANALYZE_MODEL_LOADED =
    	"Analyze model successfully loaded.";
    protected static String LOG_USE_CASES_LOADED =
    	"Use cases have been successfully loaded.";
    protected static String LOG_USE_CASE_LOADING_EXCEPTION =
    	"No available use cases or the file format is not supported.";
    protected static String LOG_USE_CASES_ADDED =
    	"Core told us that usecase was added to analyzemodel";
    protected static String FILE_MENU_CAPTION =
    	"File";
    protected static String NEW_MENU_CAPTION =
    	"New";
    protected static String OPEN_MENU_CAPTION =
    	"Open"+"...";
    protected static String LOAD_MENU_CAPTION = 
    	"Load use cases"+"...";
    protected static String SAVE_MENU_CAPTION = 
    	"Save";
    protected static String MODIFICATION_LOG_MENU_CAPTION = 
    	"Modification log..";
    protected static String SAVE_AS_MENU_CAPTION =
    	"Save as" + "...";   
    
    protected static String EXPORT_MENU_CAPTION = 
    	"Export" + "...";
    protected static String EXPORT_AS_IMAGE_MENU_CAPTION = 
    	"Export as Image"+"...";
    protected static String EXPORT_ALL_MENU_CAPTION = 
    	"Export all" + "...";
    protected static String CLEAR_MODEL_MENU_CAPTION =
    	"Clear Model"+"...";
    protected static String QUIT_MENU_CAPTION = 
    	"Quit";
    protected static String PROGRAM_MENU_CAPTION = 
    	"Program";
    protected static String SETTINGS_MENU_CAPTION =
    	"Settings" + "...";
    protected static String CHANGE_DOT_LAYOUTING_CAPTION =
    	"Use horizontal layouting";
    protected static String HELP_MENU_CAPTION =
    	"Help";
    protected static String ABOUT_MENU_CAPTION =
    	"About UCOT";
    protected static String PROGRESS_BAR_DIALOG_TITLE =
    	"Progress";
    protected static String MODIFIED_STRING = "Modified";
    protected static String UNMODIFIED_STRING = "Unmodified";
    
    protected static String NEW_FILE_MENU_ACTION = "NEW_MODEL";
    protected static String MODIFICATION_LOG_MENU_ACTION = "MODIFICATION_LOG";
    protected static String OPEN_FILE_MENU_ACTION = "OPEN_FILE";
    protected static String LOAD_FILE_MENU_ACTION = "LOAD_FILE";
    protected static String SAVE_FILE_MENU_ACTION = "SAVE_FILE";
    protected static String SAVE_AS_FILE_MENU_ACTION = "SAVE_FILE_AS";
    protected static String EXPORT_MENU_ACTION = "EXPORT";
    protected static String EXPORT_ALL_MENU_ACTION = "EXPORT_ALL";
    protected static String QUIT_MENU_ACTION = "QUIT_PROGRAM";
    protected static String CLEAR_MODEL_MENU_ACTION = "CLEAR_MODEL";
    protected static String EXPORT_AS_IMAGE_MENU_ACTION = "EXPORT_AS_IMAGE";
    protected static String SETTINGS_MENU_ACTION = "PROGRAM_SETTINGS";
    protected static String CHANGE_DOT_LAYOUTING_ACTION = "CHANGE_DOT_LAYOUTING";
    protected static String ABOUT_MENU_ACTION = "ABOUT_UCOT";
    
    /**
     * Method for changing the changes made status.
     * @param changesMade New status.
     */
    public void setChanged(boolean changesMade) {
    	this.changesMade = changesMade;
    	if ( changesMade ) changesIndicator.setText(" " + MODIFIED_STRING + " ");
    	else changesIndicator.setText(" " + UNMODIFIED_STRING + " ");
    }
    
	/**
	 * Action performed: Quit is chosen from the File menu.
	 * @param quitWithoutPrompting Indicates wether the user really
	 *   wants to quit without prompting another question to
	 *   confirm that.
	 */
	private void menuQuit(boolean quitWithoutPrompting) {
		if ( changesMade ) {
			if ( !quitWithoutPrompting && !questionDialog(QUIT_QUESTION_HEADER,
					QUIT_QUESTION).equals(JOptionPane.YES_OPTION) ) return;
			int saveChanges = Integer.parseInt(questionDialog(
			"Would you like to save the current analyze model before quitting?").
			toString());
			if ( saveChanges == JOptionPane.YES_OPTION ) {
				if (! saveModel(currentFile)){
					// No save after all this work and promises.
					// Screw u guys I am not going to home (=quit). 
					return;
				}
			}
		}
		System.out.println(LOG_PREFIX + LOG_QUIT_PROGRAM_TRIGGERED);
		System.out.println(LOG_PREFIX + LOG_QUIT_PROGRAM);
		core.shutdown();
		// TODO Change to match real shutdown routine.
	}
	
	/**
	 * Action performed: User clicks the export all menu item.
	 */
	private void menuExportAll() {
		
		URL target = GUIUtils.showSaveFileDialog(this, null, JFileChooser.DIRECTORIES_ONLY);
		if ( target == null ) return;
		
		String path = target.getPath();
		
		// Export graph image to the export directory.
		Vector<String> imageTypes = new Vector<String>();
		for ( DotPanel.ExportImageType imageType : DotPanel.ExportImageType.values() ) {
			imageTypes.add(imageType.toString());
			// Comment the following to disable exporting in all image formats.
			this.dotPanel.exportImage(new File(path + EXPORT_ALL_GRAPH_FILENAME + "." +
					imageType.toString().toLowerCase()), imageType);
		}
		
		// Remove comments to ask for just one image format for the graph.
		//String imageType = GUIUtils.showComboBoxOptionDialog("Select image format", imageTypes);
		//this.dotPanel.exportImage(new File(path + EXPORT_ALL_GRAPH_FILENAME + "." +
		//		imageType.toLowerCase()), DotPanel.ExportImageType.valueOf(imageType));
		
		Vector<String> exportTypes = new Vector<String>();
		Vector<OutputInterface> outputInterfaces = core.getOutputs();
		for ( OutputInterface outputInterface : outputInterfaces ) {
			exportTypes.add(outputInterface.getName());
		}
		
		// Export model to selected format.
		// Uncomment the following line if you want to export only one data type.
		//String exportType = GUIUtils.showComboBoxOptionDialog("Select export format", exportTypes);
		URL exportURL = null;
		
		
		for ( OutputInterface outputInterface : outputInterfaces ) {
			// Uncomment the if block if you want to export only one datatype.
			//if ( outputInterface.getName().equals(exportType) ) {
				File exportFile = new File(path + EXPORT_ALL_DATA_FILENAME + "." +
						outputInterface.getFileExtension());
				try {
					exportURL = exportFile.toURL();
				} catch (Exception e) {
					// TODO Do something here?
				}
				
				try {
					core.output(exportURL, outputInterface, core.getAnalyzeModel());
				} catch (Exception e) {
					// TODO Do something here?
				}
				break;
			//}
		}
		
		// Copy used use case files to the export directory.
		Set<URL> useCaseURLs = new HashSet<URL>();
		UseCaseCollection useCaseCollection = core.getUseCaseCollection();
		for ( int i = 0; i < useCaseCollection.getUseCaseCount(); i++ ) {
			URL useCaseFile = useCaseCollection.getUseCase(i).getUrl();
			useCaseURLs.add(useCaseFile);
		}
		
		for ( URL useCaseFile : useCaseURLs ) {
			try {
				FileTools.copyFile(useCaseFile, target);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		// Serialize analyze model.
		URL serializationURL = null;
		File serializationFile = new File(path + EXPORT_ALL_SERIALIZATION_FILENAME + "." +
				SERIALIZATION_EXTENSIONS[0].toLowerCase());
		try {
			serializationURL = serializationFile.toURL();
		} catch (Exception e) {
			// TODO Do something here?
		}
		
		core.saveAnalyzeModel(serializationURL);
		
		// Save modification log.
		try {
			core.getAnalyzeModel().getEditor().saveUpdationsToFile(new File(path, 
					EXPORT_ALL_LOG_FILENAME));
		} catch (IOException e) {
			System.out.println(StringTools.exceptionStackTrace(e));
		}
		
		// TODO Check for overwriting stuff...
	}
	
	/**
	 * Action performed: Clear model action is chosen from the File menu.
	 */
	private void menuClearModel(boolean clearWithoutPrompting) {
		System.out.println(LOG_PREFIX + LOG_CLEAR_MODEL_TRIGGERED);
		if ( CLEAR_MODEL_QUESTION_HEADER.length() <= 0 ) {
			CLEAR_MODEL_QUESTION_HEADER = DEFAULT_QUESTION_HEADER;
		}
		if ( clearWithoutPrompting || questionDialog(
				CLEAR_MODEL_QUESTION_HEADER, CLEAR_MODEL_QUESTION).
				equals(JOptionPane.YES_OPTION) ) {
			core.clearAnalyzeModel();
			setChanged(true);
		}
	}
	
	/**
	 * Action performed: User selects the new model option from the menu.
	 */
	private void menuNewModel() {
		if ( questionDialog(NEW_MODEL_QUESTION,
				NEW_MODEL_QUESTION_HEADER).equals(JOptionPane.YES_OPTION) ) {
			if (! saveModel(currentFile)){
				// Bail out if model was not saved dispite of beautifull promises.
				return ;
			}
		}
		currentFile = null;
		menuClearModel(true);
		setChanged(false);
		useCaseTree.clear();
	}
	
	/**
	 * Action performed: Open file is chosen from the File menu.
	 */
	private void menuLoadUseCaseFile() {
		System.out.println(LOG_PREFIX + LOG_FILE_LOAD_TRIGGERED);
		// TODO File filters...
		URL file = GUIUtils.showOpenFileDialog(this, null);
		
		if (null == file) return ; // Return if no selection.
		
		// If file was selected then load it.
		boolean changed = changesMade;
		core.loadUseCases(file);
		dotPanel.updateModel(core.getAnalyzeModel());
		setChanged(changed);
	}
	
	
	/**
	 * Action performed: Export is chosen from the File menu.
	 */
	private void menuExport() {		
		//System.out.println(LOG_PREFIX + LOG_FILE_LOAD_TRIGGERED);
		CustomFileFilter filter = new CustomFileFilter("GXL files",new String[] {"gxl"});
		Vector<CustomFileFilter> filters = new Vector<CustomFileFilter>();
		filters.add(filter);		
		
		URL file = GUIUtils.showSaveFileDialog(this,filters);
		if ( file == null ) return;
		
		// FIXME No hardcoding here, thank you.   vvv
		OutputInterface oi = core.getOutputs().get(0);
		try {
			core.output(file, oi, core.getAnalyzeModel());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Action performed: Export is chosen from the File menu.
	 */
	private void menuModificationLog() {
		new ModificationLogWindow(core.getAnalyzeModel());
	}
	
	/**
	 * Action performed: User wants to change program settings.
	 */
	private void menuSettings() {
		System.out.println(LOG_PREFIX + LOG_SETTINGS_TRIGGERED);
		
		// Launch settingsDialog
		SettingsDialog.showDialog(this);
		//TODO add dot panel to redraw here incase ok was pressed
		//Also inform core to apply properties		
	}
	
	/**
	 * Action performed: User wants to save picture of the model.
	 */
	private void menuExportAsImage() {
		
		// TODO Prompt user for overwriting existing files... same goes for GXL over there ^^
		
		Vector<CustomFileFilter> filters = new Vector<CustomFileFilter>();
		for (ExportImageType imagetype : ExportImageType.values()) {
			filters.add(new CustomFileFilter(imagetype.toString() + " files",
					new String[]{imagetype.toString().toLowerCase()}));
		}		
		
		URL file = GUIUtils.showSaveFileDialog(this,filters);
		if ( file == null ) return;

			
		try {
			File exportFile = new File(file.toURI());
			dotPanel.exportImage(exportFile);
		} catch (Exception e) {
			//TODO change to loggers
			e.printStackTrace();
		}
	}
	
	/**
	 * Action performed: User changes the status of the horizontal
	 * layouting checkbox.
	 */
	private void changeDotLayouting() {
		dotPanel.setHorizontalLayout(!dotPanel.getHorizontalLayout());
	}
	
	/**
	 * This method creates and initializes all required window
	 * listeners for this graphical user interface.
	 */
	private void createWindowListener() {
		/**
		 * WindowListener for all window events for the GUI window.
		 */
		windowListener = new WindowListener() {
			/**
			 * User is trying to close the window forcefully from outside
			 * the main menu.
			 */
			public void windowClosing(WindowEvent e) {
				System.out.println("Closing");
				menuQuit(true);
			}
			
			public void windowActivated(WindowEvent e) {}
			public void windowClosed(WindowEvent e) {}
			public void windowDeactivated(WindowEvent e) {}
			public void windowDeiconified(WindowEvent e) {}
			public void windowIconified(WindowEvent e) {}
			public void windowOpened(WindowEvent e) {}
			
		};
	}
	
	/**
	 * This method creates and initializes the menu listener for
	 * the program menu bar.
	 */
	private void createMenuListener() {
		// TODO Is this required?
		if ( core == null ) return;
		
		/**
		 * ActionListener for all button events within the dialog.
		 */
		menuListener = new ActionListener() {
			
			/**
			 * This method is called when an action is performed and it
			 * calls the corresponding methods to handle the action. 
			 */
			public void actionPerformed(ActionEvent e) {
				System.out.println("Action: " + e.getActionCommand());
				
				if ( LOAD_FILE_MENU_ACTION.equals(e.getActionCommand()) ) {
					menuLoadUseCaseFile(); return;
				}
				if ( EXPORT_MENU_ACTION.equals(e.getActionCommand()) ) {
					menuExport(); return;
				}
				if ( QUIT_MENU_ACTION.equals(e.getActionCommand()) ) {
					menuQuit(true); return;
				}
				if ( CLEAR_MODEL_MENU_ACTION.equals(e.getActionCommand()) ) {
					menuClearModel(false); return;
				}
				if ( SETTINGS_MENU_ACTION.equals(e.getActionCommand()) ) {
					menuSettings(); return;
				}
				if ( EXPORT_AS_IMAGE_MENU_ACTION.equals(e.getActionCommand()) ) {
					menuExportAsImage(); return;
				}
				if ( CHANGE_DOT_LAYOUTING_ACTION.equals(e.getActionCommand()) ) {
					changeDotLayouting(); return;
				}
				if ( ABOUT_MENU_ACTION.equals(e.getActionCommand()) ) {
					showAboutDialog(); return;
				}
				if ( SAVE_FILE_MENU_ACTION.equals(e.getActionCommand()) ) {
					saveModel(currentFile); return;
				}
				if ( SAVE_AS_FILE_MENU_ACTION.equals(e.getActionCommand()) ) {
					saveModel(null); return;
				}
				if ( NEW_FILE_MENU_ACTION.equals(e.getActionCommand()) ) {
					menuNewModel(); return;
				}
				if ( MODIFICATION_LOG_MENU_ACTION.equals(e.getActionCommand()) ) {
					menuModificationLog();
				}
				if ( OPEN_FILE_MENU_ACTION.equals(e.getActionCommand()) ) {
					loadModel(); return;
				}
				if ( EXPORT_ALL_MENU_ACTION.equals(e.getActionCommand()) ) {
					menuExportAll(); return;
				}
			}
		};
	}
	
	/**
	 * Constuctor for Graphical UI. This constructor sets up
	 * the whole user interface and spawns it to the screen.
	 */
	public GraphicalUI(ControlInterface controlInterface) {
		// Initialize control interface.
		this.core = controlInterface;        
		core.getAnalyzeModel().addObserver(this);
		
		try {
			loadProperties();
		} catch (IOException e) {
			System.out.println(StringTools.exceptionStackTrace(e));
		}

		// TODO parse entity types!
		// parse entity types from properties
		this.entityTypes = parseEntityTypes();
		this.setPreferredSize(new Dimension(1000, 700));
		
		// Initialize all panels.
		dotPanel = new DotPanel(this);
		dotScrollPane = new JScrollPane(dotPanel);
		dotScrollPane.setMinimumSize(new Dimension(300, 250));
		dotScrollPane.getHorizontalScrollBar().setUnitIncrement(DotPanel.SCROLL_SPEED);
		dotScrollPane.getVerticalScrollBar().setUnitIncrement(DotPanel.SCROLL_SPEED);
		this.setLayout(new BorderLayout());
		
		// create the tree form AnalyzeModel
		analyzeModelTree = new AnalyzeModelTree(this);
		modelTreeScrollPane = new JScrollPane(analyzeModelTree);
		modelTreeScrollPane.setMinimumSize(new Dimension(200, 150));
		
		createMenuListener();
		createWindowListener();
		this.addWindowListener(windowListener);
		
		useCasePanelScrollPane.setMinimumSize(new Dimension(300, 50));
		
		// Finalize analyze model tree initialization.
		analyzeModelTree.createMouseListener(core, entityPropertiesDialog);
		analyzeModelTree.setStructurePanel(dotPanel);
		
		// Create new use case tree.
		this.useCaseTree = new UseCaseTree(this.core);
		this.useCaseTree.setUseCasePanel(this.useCasePanel);
		this.useCaseTree.setDotPanel(this.dotPanel);
		
		// Put UseCaseTree into a ScrollPane.
		useCaseTreeScrollPane = new JScrollPane(this.useCaseTree);
		useCaseTreeScrollPane.setMinimumSize(new Dimension(200, 150));
		
		// Set default exit policy.
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		// Divide screen into four section with SplitPanes.
		JSplitPane treePane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, modelTreeScrollPane,
				useCaseTreeScrollPane);
		JSplitPane viewPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, dotScrollPane,
				(JComponent) useCasePanelScrollPane);
		JSplitPane sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, treePane,
				viewPane);
		
		// Finalize initialization and make window visible.
		this.getContentPane().add(statusbar, BorderLayout.SOUTH);
		this.getContentPane().add(sp, BorderLayout.CENTER);
		this.menu = createMenu();
		this.setJMenuBar(menu);
		this.setTitle(PROGRAM_TITLE);
		
		
		// Modification status indicator.
		statusbar.addColumn(changesIndicator);
		
		// Configure statusbar.
		statusbar.addColumn(dotTimer);
		dotPanel.setTimerComponent(dotTimer);
		
		this.pack();
		treePane.setDividerLocation(0.7);
		viewPane.setDividerLocation(0.7);
		sp.setDividerLocation(0.3);
		// TODO Figure out how this would actually work.
		this.setMinimumSize(new Dimension(500 + treePane.getDividerSize(),
				300 + sp.getDividerSize()));
		this.setVisible(true);
	}
	
	/**
	 * Method for spawning the about UCOT dialog.
	 */
	private void showAboutDialog() {
		AboutDialog dialog = new AboutDialog(this);
		GUIUtils.centerDialog(dialog);
		dialog.setVisible(true);
	}
	
	/**
	 * Method for creating a menu bar to the GUI.
	 * @return Program menu bar as a JMenuBar object. 
	 */
	private JMenuBar createMenu() {
		
		/* Menu will look something like this: 
		 * +------+----------+-------- 
		 * | File | Program  |
		 * =========================== 
		 * | Open | Settings | 
		 * +------+----------+ 
		 * | Exit | 
		 * +------+
		 */
		
		JMenuBar result = new JMenuBar();
		
		// FILE menu.        
		JMenu menu = new JMenu(FILE_MENU_CAPTION);
		JMenuItem menuItem;
		
		menuItem = new JMenuItem(NEW_MENU_CAPTION);
		menuItem.addActionListener(menuListener);
		menuItem.setActionCommand(NEW_FILE_MENU_ACTION);
		menu.add(menuItem);
		menuItem = new JMenuItem(OPEN_MENU_CAPTION);
		menuItem.addActionListener(menuListener);
		menuItem.setActionCommand(OPEN_FILE_MENU_ACTION);
		menu.add(menuItem);
		menuItem = new JMenuItem(LOAD_MENU_CAPTION);
		menuItem.addActionListener(menuListener);
		menuItem.setActionCommand(LOAD_FILE_MENU_ACTION);
		menu.add(menuItem);
		menu.add(new JSeparator());
		menuItem = new JMenuItem(SAVE_MENU_CAPTION);
		menuItem.addActionListener(menuListener);
		menuItem.setActionCommand(SAVE_FILE_MENU_ACTION);
		menu.add(menuItem);
		menuItem = new JMenuItem(SAVE_AS_MENU_CAPTION);
		menuItem.addActionListener(menuListener);
		menuItem.setActionCommand(SAVE_AS_FILE_MENU_ACTION);
		menu.add(menuItem);
		menuItem = new JMenuItem(EXPORT_AS_IMAGE_MENU_CAPTION);
		menuItem.addActionListener(menuListener);
		menuItem.setActionCommand(EXPORT_AS_IMAGE_MENU_ACTION);
		menu.add(menuItem);
		menuItem = new JMenuItem(EXPORT_MENU_CAPTION);
		menuItem.addActionListener(menuListener);
		menuItem.setActionCommand(EXPORT_MENU_ACTION);
		menu.add(menuItem);
		menuItem = new JMenuItem(EXPORT_ALL_MENU_CAPTION);
		menuItem.addActionListener(menuListener);
		menuItem.setActionCommand(EXPORT_ALL_MENU_ACTION);
		menu.add(menuItem);
		menu.add(new JSeparator());
		menuItem = new JMenuItem(CLEAR_MODEL_MENU_CAPTION);
		menuItem.addActionListener(menuListener);
		menuItem.setActionCommand(CLEAR_MODEL_MENU_ACTION);
		menu.add(menuItem);
		menu.add(new JSeparator());
		menuItem = new JMenuItem(MODIFICATION_LOG_MENU_CAPTION);
		menuItem.addActionListener(menuListener);
		menuItem.setActionCommand(MODIFICATION_LOG_MENU_ACTION);
		menu.add(menuItem);
		menuItem = new JMenuItem(QUIT_MENU_CAPTION);
		menuItem.addActionListener(menuListener);
		menuItem.setActionCommand(QUIT_MENU_ACTION);
		menu.add(menuItem);
		result.add(menu);
		
		// PROGRAM menu.
		menu = new JMenu(PROGRAM_MENU_CAPTION);
		
		menuItem = new JCheckBoxMenuItem(CHANGE_DOT_LAYOUTING_CAPTION);
		((JCheckBoxMenuItem)menuItem).setSelected(dotPanel.getHorizontalLayout());
		menuItem.addActionListener(menuListener);
		menuItem.setActionCommand(CHANGE_DOT_LAYOUTING_ACTION);
		menu.add(menuItem);
		menu.add(new JSeparator());
		menuItem = new JMenuItem(SETTINGS_MENU_CAPTION);
		menuItem.addActionListener(menuListener);
		menuItem.setActionCommand(SETTINGS_MENU_ACTION);
		menu.add(menuItem);
		result.add(menu);
		
		// HELP menu.
		menu = new JMenu(HELP_MENU_CAPTION);
		
		menuItem = new JMenuItem(ABOUT_MENU_CAPTION);
		menuItem.addActionListener(menuListener);
		menuItem.setActionCommand(ABOUT_MENU_ACTION);
		menu.add(menuItem);
		result.add(menu);
		
		// Menu bar successfully created!
		return result;
	}
	
	/* (non-Javadoc)
	 * @see ucot.ui.UIInterface#exportDone()
	 */
	public void exportDone() {
		System.out.println(LOG_PREFIX + LOG_EXPORT_DONE_MESSAGE);
	}
	
	/* (non-Javadoc)
	 * @see ucot.ui.UIInterface#analyzeModelLoaded()
	 */
	public void analyzeModelLoaded() {
		setChanged(true);
		((AnalyzeTreeModel)analyzeModelTree.getModel()).updateAnalyzeModel();
		//analyzeModelTree.setModel(new AnalyzeTreeModel(this));
		dotPanel.updateModel(core.getAnalyzeModel());
		System.out.println(LOG_PREFIX + LOG_ANALYZE_MODEL_LOADED);
		
		setChanged(false);
	}
	
	/* (non-Javadoc)
	 * @see ucot.ui.UIInterface#useCasesLoaded()
	 */
	public void useCasesLoaded() {
		setChanged(true);
		System.out.println(LOG_PREFIX + LOG_USE_CASES_LOADED);
		UseCaseCollection collection = core.getUseCaseCollection();
		try {
			if (collection.getUseCaseCount() > 0) {
				// Ask user which use case he or she desires to load.
				//			UseCase uc = useCaseSelector.selectUseCase(collection);
				//			core.addToAnalyzeModel(uc);
				
			} else {
				throw new
				Exception(LOG_USE_CASE_LOADING_EXCEPTION);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/* (non-Javadoc)
	 * @see ucot.ui.UIInterface#useCaseAdded()
	 */
	public void useCaseAdded() {
		setChanged(true);
		System.out.println(LOG_PREFIX + LOG_USE_CASES_ADDED);
	}
	
	/* (non-Javadoc)
	 * @see ucot.ui.UIInterface#setControlInterface(ucot.core.ControlInterface)
	 */
	public void setControlInterface(ControlInterface a) {
		core = a;
	}
	
	/* (non-Javadoc)
	 * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
	 */
	public void update(Observable o, Object arg) {
		setChanged(true);
	}
	
	/**
	 * This method serializes the current status of the project.
	 * @param target Path to the file used for saving. If this
	 *   is null, then a save file dialog will be spawned.
	 * @return true if model was saved, false otherwhise.
	 */
	public boolean saveModel(URL target) {
		if ( target == null ) {
			Vector<CustomFileFilter> fileFilters = new Vector<CustomFileFilter>();
			fileFilters.add(new
					CustomFileFilter(SERIALIZATION_DESCRIPTION,
							SERIALIZATION_EXTENSIONS));
			// Filename was not given so we need to ask the user about it.
			target = GUIUtils.showSaveFileDialog(this, fileFilters);
			if ( target == null ) return false;
			// Now check wether the file already exists.
			File file = new File(target.getFile());
			// File exists, so we need to confirm the overwriting of it.
			if ( file.exists() ) {
				int overwrite = Integer.parseInt(questionDialog(
						"File " + target.getFile() + " already exists.\n" +
				"Do you want to replace it?").toString());
				if ( overwrite == JOptionPane.NO_OPTION ) return false;
			}
		}
		
		setChanged(false);
		
		core.saveAnalyzeModel(target);
		currentFile = target;
		return true;
	}
	
	private Vector<String> parseEntityTypes() {
		
		Vector<String> result = new Vector<String>();
		String typeString = this.properties.getProperty(PROPERTY_ENTITY_TYPES);
		
		// if there was no types in properties
		// no need for parsing
		if (!"".equals(typeString)) {
			
			String[] types = typeString.split(", ");
			
			for (int i = 0; i < types.length; i++) {
				result.add(types[i]);
			}
			
		}
		
		return result;
	}
	
	/**
	 * Default save method. The previous file will be overwritten.
	 */
	public void saveModel() {
		saveModel(currentFile);
	}
	
	/**
	 * Default load model method.
	 */
	public void loadModel() {
		Vector<CustomFileFilter> fileFilters = new Vector<CustomFileFilter>();
		fileFilters.add(new	CustomFileFilter(SERIALIZATION_DESCRIPTION,
				SERIALIZATION_EXTENSIONS));
		URL target = GUIUtils.showOpenFileDialog(this, fileFilters);
		// Check file's existance.
		if ( target == null ) return;
		File file = new File(target.getFile());
		if ( !file.exists() ) {
			printError("File " + target.getFile() + " does not exist!");
			return;
		}
		
		setChanged(false);
		currentFile = target;
		
		try {
			core.loadAnalyzeModel(target);
		} catch (Exception e) {
			printError(e.getMessage());
		}
	}
	
	public ProgressBarInterface getProgressBar() {
		// Initialize progress bar dialog.
		if ( progressBarDialog == null ) { 
			progressBarDialog = new ProgressBarDialog(this, PROGRESS_BAR_DIALOG_TITLE);
		}
		return progressBarDialog;
	}
	
	public void printError(String errorMessage) {
		GUIUtils.printError(this,errorMessage);		
	}
	
	public void printWarning(String warningMessage, String warningTitle) {
		GUIUtils.printWarning(this,warningMessage, warningTitle);
	}
	
	public void printWarning(String warningMessage) {
		GUIUtils.printWarning(this,warningMessage);
	}
	
	public Object questionDialog(String question, String title, int options) {
		return GUIUtils.questionDialog(this,question,title,options);
	}
	
	public Object questionDialog(String question, String title) {
		return GUIUtils.questionDialog(this,question,title);
	}
	
	public Object questionDialog(String question) {
		return GUIUtils.questionDialog(this,question);
	}
	
	public void printError(String errorMessage, String errorTitle) {
		GUIUtils.printError(this,errorMessage, errorTitle);
		
	}
	
	public void setDisabled(boolean disabled) {
		this.analyzeModelTree.setEnabled(!disabled);
		this.useCaseTree.setEnabled(!disabled);
		this.menu.setEnabled(!disabled);
	}
	
	/**
	 * Method for creating the URL from the properties file, which
	 * is the same as the class name with an .xml extension.
	 * @return URL to the properties file.
	 */
	private URL getPropertiesURL() {
		String filename = this.getClass().toString();
		filename = filename.substring(filename.lastIndexOf('.') + 1) + ".xml";
		File propertiesFile = new File(filename);
		try {
			propertiesFile.createNewFile();
		} catch (IOException e) {}
		
		try {
			return propertiesFile.toURL();
		} catch (MalformedURLException e) {
			return null;
		}
	}
	
	/* (non-Javadoc)
	 * @see ucot.ModulePropertyInterface#getProperties()
	 */
	public synchronized Properties getProperties() {
		return properties;
	}
	
	/* (non-Javadoc)
	 * @see ucot.ModulePropertyInterface#setProperties(java.util.Properties)
	 */
	public synchronized void setProperties(Properties properties) {
		this.properties = PropertiesTools.merge(properties, this.properties);
	}
	
	/* (non-Javadoc)
	 * @see ucot.ModulePropertyInterface#applyProperties()
	 */
	public synchronized void applyProperties() throws BadPropertyValueException {
		// TODO Should this do something else?
		dotPanel.colorTheme.changeColor(properties.
				getProperty(PROPERTY_DOT_COLOR));
		dotPanel.colorTheme.changeHighlight(properties.
				getProperty(PROPERTY_DOT_HIGHLIGHT_COLOR));
		dotPanel.updateModel(core.getAnalyzeModel());
	}
	
	/* (non-Javadoc)
	 * @see ucot.ModulePropertyInterface#saveProperties()
	 */
	public synchronized void saveProperties() throws IOException {
		PropertiesTools.saveProperties(properties, this.propertiesURL);
	}
	
	/* (non-Javadoc)
	 * @see ucot.ModulePropertyInterface#loadProperties()
	 */
	public synchronized void loadProperties() throws IOException {
		properties = PropertiesTools.merge(PropertiesTools.loadProperties(
				loadDefaultProperties().propertyNames(),
				this.propertiesURL), properties);
	}
	
	/* (non-Javadoc)
	 * @see ucot.ModulePropertyInterface#loadDefaultProperties()
	 */
	public Properties loadDefaultProperties() {
		Properties result = new Properties();
		result.setProperty(PROPERTY_DOT_PATH, "./dot/dot.exe");
		result.setProperty(PROPERTY_EPS_TO_PDF_PATH, "./dot/epstopdf.exe");
		result.setProperty(PROPERTY_TEMP_INPUT_FILE, "temp.dot");
		result.setProperty(PROPERTY_TEMP_OUTPUT_FILE, "graph.png");
		result.setProperty(PROPERTY_TEMP_EPS_FILE, "temp.eps");
		result.setProperty(PROPERTY_DOT_COLOR, "BLACK_ON_WHITE");
		result.setProperty(PROPERTY_DOT_HIGHLIGHT_COLOR, "GREEN");
		result.setProperty(PROPERTY_ENTITY_TYPES, "");
		result.setProperty(PROPERTY_GRAPH_FONT_SIZE, "10");
		// TODO Set (other) default properties here.
		return result;
	}
	
	public ControlInterface getControlInterface() {
		return core;
	}
	
	public DotColorTheme getColorTheme() {
		return dotPanel.colorTheme;
	}
}
