package ucot.ui.gui.dialog;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

import ucot.Messages;
import ucot.core.ControlInterface;

/**
 * <pre>public class <b>MergeEntitiesDialog</b> extends JDialog</pre>
 * 
 * <p>Dialog extending <code>JDialog</code> component for user to 
 * select wich entities he/she wants to merge.</p>
 * 
 * <p>If user selects the entity to merge with, dialog merges entity
 * given in constructor with the selected entity. Name of the merged entity
 * is name of the entity user selected from combobox</p>
 * 
 * @see JDialog
 * 
 * @author ilanliuk
 *
 */
public class MergeEntitiesDialog extends JDialog {

    private ControlInterface core;
    private String entity;
    
    //
    // Dialog components
    JLabel label;
    JComboBox selector;
    JButton okButton;
    JButton cancelButton;
    // Dialog components
    //

    //
    // Action commands
    private final static String OK_BUTTON_ACTION = "OK_BUTTON"; //$NON-NLS-1$
    private final static String CANCEL_BUTTON_ACTION = "CANCEL_BUTTON"; //$NON-NLS-1$
    
    private ActionListener buttonListener = new ActionListener() {
    
        public void actionPerformed(ActionEvent e) {
    
            //
            // check what action happened
            
            //
            // if user pressed OK-button merge selected entities
            if ( OK_BUTTON_ACTION.equals(e.getActionCommand()) ) {
                
                String entityToMerge = getSelected();
                
                if (entityToMerge != null){
                	Set<String> mergeList = new HashSet<String>();
                	mergeList.add(entity);
                	
                	core.getAnalyzeModel().getEditor().mergeEntity(entityToMerge,mergeList);
                	
                }
                
                MergeEntitiesDialog.this.setVisible(false);
                
            }
            
            //
            // if user pressed Cancel-button just close the dialog
            // without doing nothing
            if ( CANCEL_BUTTON_ACTION.equals(e.getActionCommand()) ) {
                
                MergeEntitiesDialog.this.setVisible(false);
                
            }
        }
    };
    
    /**
     * <p>Default constructor for MergeEntitiesDialog</p>
     * @param owner the <code>Frame</code> from which the dialog is displayed.
     * @param core <code>ControllInterface</code>.
     * @param entity Name of the entity to merge.
     * @throws HeadlessException if GraphicsEnvironment.isHeadless()
     * returns true.
     */
    public MergeEntitiesDialog(Frame owner, ControlInterface core, String entity) throws HeadlessException {
        super(owner, Messages.getString("MergeEntitiesDialog.DialogTitle"), true); //$NON-NLS-1$

        this.core = core;
        this.entity = entity;
                
        // initialize label
        // FIXME: name of the entity to be merged should be somehow highlighted.
        // one solution could be using 3 JLabels "Merge", "entity name" and "with".
        JLabel label = new JLabel(Messages.getFormatedMessage("MergeEntitiesDialog.MERGE_LABEL", entity)); //$NON-NLS-1$
        
        JPanel labelPane = new JPanel();
        labelPane.setLayout( new FlowLayout(FlowLayout.LEFT, 10, 10) );
        labelPane.add(label);
        
        // initialize combobox 
        selector = getComboBox();
        
        JPanel selectorPane = new JPanel();
        selectorPane.setLayout( new FlowLayout(FlowLayout.CENTER, 10, 0) );
        selectorPane.add(selector);
        
        //
        // create and initialize buttons
        JPanel buttonPane = new JPanel();
        buttonPane.setLayout(new FlowLayout(FlowLayout.CENTER, 10, 10));
        okButton = new JButton(Messages.getString("MergeEntitiesDialog.ButtonOKCaption")); //$NON-NLS-1$
        okButton.setActionCommand(OK_BUTTON_ACTION);
        okButton.addActionListener(buttonListener);
        
        cancelButton = new JButton(Messages.getString("MergeEntitiesDialog.ButtonCancelCaption")); //$NON-NLS-1$
        cancelButton.setActionCommand(CANCEL_BUTTON_ACTION);
        cancelButton.addActionListener(buttonListener);
        
        buttonPane.add(okButton);
        buttonPane.add(cancelButton);

        // add components into dialog
        this.getContentPane().setLayout( new BorderLayout() );
        this.getContentPane().add(labelPane, BorderLayout.NORTH);
        this.getContentPane().add(selectorPane, BorderLayout.CENTER);
        this.getContentPane().add(buttonPane, BorderLayout.SOUTH);
        
        this.pack();
    }
    
    /**
     * <p>Returns <code>JComboBox</code> with entities from
     * <code>AnalyzeModel</code> of given <code>ControlInterface</code>
     * as selections.</p>
     * <p>The entity to merge is not in selections.</p>
     * @return <code>JComboBox</code>
     */
    private JComboBox getComboBox() {
        JComboBox result = new JComboBox();
        
        List<String> entities = new Vector<String>();
        entities.addAll( core.getAnalyzeModel().getEditor().getEntityNames() );
        
        // sort entities to alphabetic order
        Collections.sort(entities);
        
        for (String ent : entities) {
            
            // we  bropably don't want to merge entity
            // with itself.
            if (ent.equals(entity)) {
                continue;
            }
            result.addItem(ent);
        }
        
        return result;
    }
    
    /**
     * <p>Returns selected item from selector <code>JComboBox</code></p>.
     * @return Selected item from selector <code>JComboBox</code>.
     */
    private String getSelected() {
        return selector.getSelectedItem().toString();
    }
    
    /**
     * <p>Initializes new instance of <code>MergeEntitiesDialog</code> and shows it.</p>
     * @param core <code>ControlInterface</code>
     * @param entity Name of the entity to merge.
     */
    public static void showDialog(ControlInterface core, String entity) {
        
    	if (core == null) return;
    	if (entity == null || entity.equals("")) return;
    	
        // create new dialog
        MergeEntitiesDialog dialog = new MergeEntitiesDialog(null, core, entity);
        
        // and throw it on the screen
        dialog.setVisible(true);
    }
}
