package ucot.ui.gui.dot;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collections;
import java.util.HashSet;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;
import java.util.Stack;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.JLabel;
import javax.swing.JPanel;

import ucot.model.AnalyzeModel;
import ucot.model.AnalyzeModelException;
import ucot.model.ModelEditor;
import ucot.model.Updation;
import ucot.model.Updation.UpdationType;
import ucot.model.object.ObjectAnalyzeModel;
import ucot.ui.gui.GraphicalUI;

/**
 * A dot panel class inherited from JPanel class which is used for
 * drawing dot graphs from the given analyze model. This class
 * implements the observer interface which allows objects to keep
 * track on the current status of this panel.
 * 
 * This panel also observes the given analyze model and updates
 * the graph when the model changes.
 * 
 * There is also a method which allows highlighting any submodels
 * from the given analyze model.
 * 
 * @author ilanliuk, tujupien, vevijopi.
 */
public class DotPanel extends JPanel implements Observer {
	
	/**
	 * This class is a container for a dot job which includes all
	 * required information for rendering both highlighted and
	 * regular dot graphs. A container like this is required when
	 * dot execution is done in threads.
	 */
	private class DotJob {
		private AnalyzeModel analyzeModel = new ObjectAnalyzeModel();
		private AnalyzeModel highlightModel = new ObjectAnalyzeModel();
		
		private Set<String> entitiesToBeHighlighted =
	    	new HashSet<String>();
		private Set<String> entitiesInCurrentModel = 
	    	
	    	new HashSet<String>();
		private Set<String> entitiesRequestedForHighlighting = 
	    	new HashSet<String>();
		private Set<Vector<String>> parentsToBeHighlighted = 
	    	new HashSet<Vector<String>>();
		private Set<Vector<String>> parentsInCurrentModel = 
	    	new HashSet<Vector<String>>();
		private Set<Vector<String>> parentsRequestedForHighlighting = 
	    	new HashSet<Vector<String>>();
		private Set<Vector<String>> methodsToBeHighlighted = 
	    	new HashSet<Vector<String>>();
		private Set<Vector<String>> methodsInCurrentModel = 
	    	new HashSet<Vector<String>>();
		private Set<Vector<String>> methodsRequestedForHighlighting = 
	    	new HashSet<Vector<String>>();
		private Set<Vector<String>> attributesToBeHighlighted =
	    	new HashSet<Vector<String>>();
		private Set<Vector<String>> attributesInCurrentModel = 
	    	new HashSet<Vector<String>>();
		private Set<Vector<String>> attributesRequestedForHighlighting = 
	    	new HashSet<Vector<String>>();
	}
	
	private class ExportJob {
		private BufferedImage graph = new BufferedImage(1, 1, BufferedImage.TYPE_INT_RGB);
		private ExportImageType imageType = DEFAULT_EXPORT_IMAGE_TYPE;
		private File targetFile = new File(owner.getProperties().getProperty(
				GraphicalUI.PROPERTY_TEMP_OUTPUT_FILE) + ".tmp");
		
	}
	
	public static final long serialVersionUID = 3899637467435823526L;
	private static boolean THREAD_RUNNING = true;
	private static Stack<DotJob> DOT_JOB_STACK = new Stack<DotJob>();
	private static Thread EXECUTING_DOT_THREAD = null;
	private static Stack<ExportJob> EXPORT_JOB_STACK = new Stack<ExportJob>();
	private static Thread EXECUTING_EXPORT_THREAD = null;

	//TODO comment me
	public enum ExportImageType {PNG,JPG,EPS,PDF,SVG};
	public static final ExportImageType DEFAULT_EXPORT_IMAGE_TYPE = ExportImageType.PNG;
    
    protected long timeElapsed;
    protected JLabel timerComponent;
    
    //private AnalyzeModel analyzeModel;
    protected int fontSize = 5;
    
    public DotColorTheme colorTheme = new DotColorTheme();
    public final static int SCROLL_SPEED = 20;
	
    private BufferedImage modelImage = null;
    protected GraphicalUI owner;
    
    private boolean horizontalLayout = true;
    private boolean isUpdating;
    
    /**
     * Constructor for DotPanel. This constructor adds an observer
     * to the analyze model and sets its own status to updating.
     * @param analyzeModel analyzeModel to use with this DotPanel.
     */
    public DotPanel(GraphicalUI owner) {
    	this.owner = owner;
        owner.getControlInterface().getAnalyzeModel().addObserver(this);
        this.isUpdating = true;
    }
    
    /**
     * Sets wether or not the dot should use the horizontal layout
     * for the entities and their relationships.
     * @param horizontalLayout New value for horizontal layout,
     *   true means yay for horizontal layouting.
     */
    public void setHorizontalLayout(boolean horizontalLayout) {
    	this.horizontalLayout = horizontalLayout;
    	highlight(null);
    }
    
    /**
     * Returns the current value of horizontal layouting.
     * @return True if horizontal layout is in use, else false.
     */
    public boolean getHorizontalLayout() {
    	return this.horizontalLayout;
    }

    /* (non-Javadoc)
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update(Observable observableObject, Object updationArg) {
        if ( !(updationArg instanceof Updation) )
            return;
        
        // Change updation status.
        Updation updation = (Updation) updationArg;
        if ( updation.getType() == UpdationType.MODIFICATION_STARTED ) {
            this.isUpdating = false;
        }
        if ( updation.getType() == UpdationType.READY ||
        		updation.getType() == UpdationType.CLEAR ) {
            this.isUpdating = true;
        }
        
        // Update graph if required.
        if ( this.isUpdating ) {
            this.updateModel(owner.getControlInterface().getAnalyzeModel());
        }
    }
    
    /**
     * This method writes the graph given as a buffered image to the
     * given file in the format specified by imageType.
     * @param graph Graph to be written.
     * @param target Target path for the graph file.
     * @param imageType File format.
     * @throws IOException If something goes wrong with the writing,
     *   then an exception is thrown.
     */
    private void saveImage(BufferedImage graph, File target,
    		ExportImageType imageType) throws IOException {
    	ImageIO.write(graph, imageType.toString(), target);
    }
    
    /**
     * This method generates a new graph from the analyze model and
     * exports it to the given target file in a format specified by
     * imageType.
     * @param target File path where to save the graph.
     * @param imageType File format of the file.
     * @throws IOException If something goes wrong with the writing,
     *   then an exception is thrown.
     */
    private void saveExport(File target, ExportImageType imageType) throws IOException {
    	DotJob job = new DotJob();
    	try {
    		job.analyzeModel.getEditor().merge(owner.getControlInterface().getAnalyzeModel());
    	} catch (AnalyzeModelException e) {
    		e.printStackTrace();
    	}
    	File dotFile = createDotFile(job);
    	String args = "";
    	
    	switch (imageType) {
			case EPS:
				args = "-Gsize=8,10.7 -Gmargin=0 -Nfontsize=7 -Efontsize=6 " +
						"-Gpagedir=TL -Gremincross=true -Gconcentrate=true -Tps2";
				runDot(dotFile, target, args);
				break;
			case PDF:
				File epsImage = new File(this.owner.getProperties().
	            		getProperty(GraphicalUI.PROPERTY_TEMP_EPS_FILE));
				args = "-Gsize=8,10.7 -Gmargin=0 -Nfontsize=7 -Efontsize=6 " +
						"-Gpagedir=TL -Gremincross=true -Gconcentrate=true -Tps2";
				runDot(dotFile, epsImage, args);
				convertEPSToPDF(epsImage, target);
				break;
			case SVG:
				runDot(dotFile, target, "-Tsvg");
				break;
			case JPG:
				runDot(dotFile, target, "-Tjpg");
				break;
			default: break;
		}
    }
    
    /**
     * This method should be used inside a new thread. Once started,
     * this method will keep on running until the DotPanel gets finalized
     * and during its lifetime, this method will perform all graph
     * exporting scheduled for DotPanel.
     */
    private void executeExport() {
    	while ( THREAD_RUNNING ) {
			if ( !EXPORT_JOB_STACK.isEmpty() ) {

				ExportJob job = null;
				
				synchronized (EXPORT_JOB_STACK) {
					job = EXPORT_JOB_STACK.pop();
				}
				
				if ( job == null ) continue;
				
				try {
					// PNG and JPG exports are done in the same way. They're
					// just being written to the disk as-is.
			        if ( job.imageType == ExportImageType.PNG ) {
			        	saveImage(job.graph, job.targetFile, job.imageType);
			        }
			        // Others use a different approach and must be generated
			        // by dot instead of easily just saving the image.
			        else {
			        	saveExport(job.targetFile, job.imageType);
			        }
				} catch (IOException e) {
					e.printStackTrace();
				}
			} else {
				try {
					synchronized (EXECUTING_EXPORT_THREAD) {
						EXECUTING_EXPORT_THREAD.wait();
					}
				} catch (Exception e) { e.printStackTrace(); }
			}
		}
    }
    
    /**
     * This method starts a new thread for export actions if
     * one has not been started yet. In each case a new export
     * job is pushed to the top of the export job stack and
     * the thread is notified about this action, which wakes
     * the thread and it will start doing the topmost job from
     * the stack when it wakes up.
     * @param target Target where the file will be saved.
     * @param imageFormat Format for the file.
     */
    public void exportImage(File target, ExportImageType imageType) {
    	// if no usecases have been added into 
        // analyzemodel modelImage is null..
    	if ( target == null || modelImage == null ) return;
    	
    	ExportJob job = new ExportJob();
    	job.graph = modelImage;
    	job.imageType = imageType;
    	job.targetFile = target;
    	
    	synchronized (EXPORT_JOB_STACK) {
    		EXPORT_JOB_STACK.push(job);
    	}
    	if ( EXECUTING_EXPORT_THREAD == null ) {
    		EXECUTING_EXPORT_THREAD = new Thread(new Runnable() {
				public void run() {
					executeExport();
				}        	
	        });
    		EXECUTING_EXPORT_THREAD.start();
    	} else {
    		synchronized (EXECUTING_EXPORT_THREAD) {
    			EXECUTING_EXPORT_THREAD.notify();
    		}
    	}
    }
    
    /**
     * Saves current image to given file in format
     * that is tried to guess from filename. 
     * @param target File pointing the saving destination.
     */
    public void exportImage(File target) {
        
        // Strip extension from filename
        String filename = target.getName();
        String extension = filename.substring(filename.lastIndexOf(".") + 1);
        boolean imagetypeFound = false;
        
        // try to resolve in wich imagetype we should save the image.
        for (ExportImageType imagetype : ExportImageType.values()) {
            if (extension.equals(imagetype.toString().toLowerCase())) {
                exportImage(target, imagetype);
                imagetypeFound = true;
                continue;
            }
        }
        if (!imagetypeFound) {
            throw new Error("No ExportImageType found for file extension of \""
            		+ target.getName() + "\".");
        }
    }
    
    /**
     * Converts eps file to pdf
     * @param epsImage eps file to read
     * @param target File path where to save.
     * @return true if everything went ok
     */
    private void convertEPSToPDF(File epsImage, File target) throws IOException {
    	File epstopdfExecutable = new File(this.owner.getProperties().
    			getProperty(GraphicalUI.PROPERTY_EPS_TO_PDF_PATH));
    	String executeCommand = epstopdfExecutable.getAbsolutePath() +
    		" --outfile=" + target.getAbsolutePath() + " " + epsImage.getAbsolutePath();
    	
    	// Run EPS2PDF executable.
        Process epsToPDFProcess = null;
        
        epsToPDFProcess = Runtime.getRuntime().exec(executeCommand);

        if ( epsToPDFProcess == null ) return;

        try {
        	epsToPDFProcess.waitFor();
        } catch (InterruptedException e) {
            System.err.println(e.getMessage());
            return;
        }
    }   

    
    /**
     * This method runs the dot executable with the given dot input file
     * and returns the location of the image file.
     * @param dotInputFile Input file for dot executable.
     * @param outputFile Path to the output file.
     * @param args Command line arguments for dot.
     * @return File where the image is located. Null is returned if
     *   something went wrog.
     */
    private File runDot(File dotInputFile, File outputFile, String args) {
        // Ensure that we can find the dot executable.
        File dotExecutable = new File(this.owner.getProperties().
        		getProperty(GraphicalUI.PROPERTY_DOT_PATH));
        if ( !dotExecutable.exists() ) {
            System.err.println("Error in DotPanel.java: can't find the dot.exe in given path '"
                            + dotExecutable.getAbsolutePath() + "'");
            return null;
        }

        // TODO Debug this on Linux.
        // Get the output from dot executable as a PNG image.
        String executeCommand = dotExecutable.getAbsolutePath() + " -o " +
        	outputFile.getAbsolutePath() + " " + args + " " +
        	dotInputFile.getAbsolutePath();

        // Run dot executable.
        Process dotProcess = null;
        try {
            dotProcess = Runtime.getRuntime().exec(executeCommand);
        } catch (IOException e) {
            System.err.println(e.getMessage());
            outputFile = null;
        }

        if (dotProcess == null)
            return null;

        try {
            dotProcess.waitFor();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        
        return outputFile;
    }
    
    /**
     * Dot runner which uses the default temporary output file as
     * a target for the file.
     * @param dotInputFile Input file for dot executable.
     * @param args Command line arguments for dot.
     * @return File where the image is located. Null is returned if
     *   something went wrog.
     */
    private File runDot(File dotInputFile, String args) {
    	File outputFile = new File(this.owner.getProperties().
        		getProperty(GraphicalUI.PROPERTY_TEMP_OUTPUT_FILE));
    	return runDot(dotInputFile, outputFile, args);
    }
    
    /**
     * Default dot runner. This will create the dot graph with default
     * settings (PNG image with the paths defined in settings XML).
     * @param dotInputFile Input file for dot executable.
     * @return File where the image is located. Null is returned if
     *   something went wrog.
     */
    private File runDot(File dotInputFile) {
    	return runDot(dotInputFile, "-Tpng");
    }
    
    /**
     * Method for writing the methods of one entity to the dot file.
     * @param job Defines the entities to be written.
     * @param writer BufferedWriter to use for writing.
     * @param entity Name of the entity whose methods are being written.
     * @param entityLabel Label to show for this entity in the graph.
     * @throws IOException If something goes wrong an exception is thrown.
     */
    private void writeEntityMethods(DotJob job, BufferedWriter writer,
    		String entity, String entityLabel) throws IOException {
    	ModelEditor editor = job.analyzeModel.getEditor();
    	Set<String> methods = editor.getMethodNames(entity);
        
        try {
	        for ( String method : methods ) {
	            Set<String> targets = editor.getEntitiesInfluencedByMethod(
	                    entity, method);
	
	            for ( String target : targets ) {
	            	String targetType = job.analyzeModel.getEditor().
	            	    	getEntityType(target);
	            	if ( targetType != null ) {
	            		target += "\n" + targetType;
	                }
	            	entityLabel = entityLabel.replace("<", "&lt;");
	            	entityLabel = entityLabel.replace(">", "&gt;");
	            	target = target.replace("<", "&lt;");
            		target = target.replace(">", "&gt;");
	                writer.write("<" + entityLabel + "> -> <" + target
	                        + "> [label=\"" + method
	                        + "\", arrowhead=\"vee\", ");
	                Vector<String> compare = new Vector<String>();
	                compare.add(entity);
	                compare.add(method);
	                compare.add(target);
	                writer.write(colorTheme.getColorString(job.methodsToBeHighlighted
	                		!= null && job.methodsToBeHighlighted.contains(compare)));
	                writer.write(", fontsize=\"" + fontSize + "\"];");
	                writer.newLine();
	            }
	        }
        } catch (IOException e) {
        	throw e;
        }
    }
    
    /**
     * Method for writing the attributes of one entity to the dot file.
     * @param job Defines the entities to be written.
     * @param writer BufferedWriter to use for writing.
     * @param entity Name of the entity whose attributes are being written.
     * @param entityLabel Label to show for this entity in the graph.
     * @throws IOException If something goes wrong an exception is thrown.
     */
    private void writeEntityAttributes(DotJob job, BufferedWriter writer,
    		String entity, String entityLabel) throws IOException {
    	ModelEditor editor = job.analyzeModel.getEditor();
        Set<String> attributes = editor.getAttributeNames(entity);
        for ( String attribute : attributes ) {
        	String attributeType = job.analyzeModel.getEditor().getEntityType(attribute);
        	if ( attributeType != null ) {
        		attribute += "\n" + attributeType;
            }
        	entityLabel = entityLabel.replace("<", "&lt;");
        	entityLabel = entityLabel.replace(">", "&gt;");
    		attribute = attribute.replace("<", "&lt;");
    		attribute = attribute.replace(">", "&gt;");
    		try {
	        	writer.write("<" + entityLabel + "> -> <" + attribute + "> [taillabel=\"" +
	        			editor.getAttributeFromCardinal(entity, attribute)
	        			+"\", headlabel=\""+editor.getAttributeToCardinal(entity, attribute)+
	        			"\", arrowtail=\"diamond\", arrowsize=\"1.3\", labeldistance=\"1.7\""+
	        			", arrowhead=\"none\", ");
    		} catch (AnalyzeModelException e) {
    			e.printStackTrace();
    		}
        	Vector<String> compare = new Vector<String>();
        	compare.add(entity);
        	compare.add(attribute);
        	writer.write(colorTheme.getColorString(job.attributesToBeHighlighted
        			!= null && job.attributesToBeHighlighted.contains(compare)));
        	writer.write(", fontsize=\"" + fontSize + "\"];");
        	writer.newLine();
        }
    }
    
    /**
     * Method for writing the parents of one entity to the dot file.
     * @param job Defines the entities to be written.
     * @param writer BufferedWriter to use for writing.
     * @param entity Name of the entity whose parents are being written.
     * @param entityLabel Label to show for this entity in the graph.
     * @throws IOException If something goes wrong an exception is thrown.
     */
    private void writeEntityParents(DotJob job,	BufferedWriter writer,
    		String entity, String entityLabel) throws IOException {
    	Set<String> parents = job.analyzeModel.getEditor().getParents(entity);
        for ( String parent : parents ) {
        	String parentType = job.analyzeModel.getEditor().getEntityType(parent);
			if ( parentType != null ) {
				parent += "\n" + parentType;
		    }
			entityLabel = entityLabel.replace("<", "&lt;");
        	entityLabel = entityLabel.replace(">", "&gt;");
			parent = parent.replace("<", "&lt;");
			parent = parent.replace(">", "&gt;");
        	writer.write("<" + parent + "> -> <" + entityLabel + 
        			"> [arrowtail=\"onormal\", arrowhead=\"none\", arrowsize=\"1.5\", ");
        	Vector<String> compare = new Vector<String>();
        	compare.add(entity);
        	compare.add(parent);
        	writer.write(colorTheme.getColorString(job.parentsToBeHighlighted
        			!= null && job.parentsToBeHighlighted.contains(compare)));
        	writer.write(", fontsize=\"" + fontSize + "\"];");
        	writer.newLine();
        }
    }
    
    /**
     * Method for writing all entities to the dot file with all their
     * attribute, method and parent relationships.
     * @param job Defines the entities to be written.
     * @param writer BufferedWriter to use for writing.
     * @throws IOException An exception is thrown if something goes wrong.
     */
    private void writeEntities(DotJob job, BufferedWriter writer) throws IOException {
        Set<String> entityNames = job.analyzeModel.getEditor().getEntityNames();
        Vector<String> entityNamesOrdered = new Vector<String>();
        entityNamesOrdered.addAll(entityNames);
        Collections.sort(entityNamesOrdered);
        if ( !horizontalLayout ) Collections.reverse(entityNamesOrdered);
        
        try {
	    	for ( String entity : entityNamesOrdered ) {
	        	String entityLabel = job.analyzeModel.getEditor().getEntityType(entity);
		        if ( entityLabel != null ) {
		           	entityLabel = entity + "\n" + entityLabel;
		        } else {
		        	entityLabel = entity;
		        }
	        	entityLabel = entityLabel.replace("<", "&lt;");
	        	entityLabel = entityLabel.replace(">", "&gt;");
	            writer.write("<" + entityLabel + "> [");
	            writer.write(colorTheme.getColorString(job.entitiesToBeHighlighted
	            		!= null && job.entitiesToBeHighlighted.contains(entity)));
	            writer.write(", fontsize=\"" + fontSize + "\"];");
	            writer.newLine();
	            
	            writeEntityMethods(job, writer, entity, entityLabel);
	            writeEntityAttributes(job, writer, entity, entityLabel);
	            writeEntityParents(job, writer, entity, entityLabel);
	        }
        } catch (IOException e) {
        	throw e;
        }
    }
    
    /**
     * Writes the analyze model to an external file in dot syntax.
     * @param job Dot job which defines the model to be drawn.
     * @return File containing the analyze model in dot syntax.
     *   Null is returned if something went wrong.
     */
    private File createDotFile(DotJob job) {

        File result = new File(this.owner.getProperties().
        		getProperty(GraphicalUI.PROPERTY_TEMP_INPUT_FILE));

        // Initialize stream to file for writing.
        BufferedWriter writer = null;
        try {
            writer = new BufferedWriter(new FileWriter(result));
            
            try {
            	fontSize = Integer.parseInt(this.owner.getProperties().
            		getProperty(GraphicalUI.PROPERTY_GRAPH_FONT_SIZE));
            } catch (Exception e) {}
            if ( fontSize < 5 ) fontSize = 5;
            else if ( fontSize > 100 ) fontSize = 100;
            
            // Write starting tag into file and graph-wide attributes.
            writer.write("digraph G { [" + colorTheme.getBackgroundColorString() + 
            		", rankdir=\"" + (horizontalLayout ? "TB" : "LR" ) + "\"" +
            		", fontsize=\"" + fontSize + "\"" +
            		"]");
            writer.newLine();
            
            // Write the entities and their relationships to the dot file.
            writeEntities(job, writer);

            // Write end tag into file.
            writer.write("}");
            writer.newLine();

            writer.flush();
            writer.close();

        } catch (IOException e) {
            System.err.println(e.getMessage());
            return null;
        }

        return result;
    }
    
    /**
     * Method for mapping the current analyze model. This is required to
     * figure out differences with the submodel that is required to be
     * highlighted.
     * @param job Dot job where mapping will be done.
     */
    private void mapCurrentModel(DotJob job) {
    	for ( String entity : job.entitiesInCurrentModel ) {
    		for ( String method : job.analyzeModel.getEditor().getMethodNames(entity) ) {
    			for ( String target : job.analyzeModel.getEditor().
    					getEntitiesInfluencedByMethod(entity, method) ) {
    				// Method vector stores the method name plus its owner and
    				// target entities.
    				Vector<String> methodVector = new Vector<String>();
    				methodVector.add(entity);
    				methodVector.add(method);
    				methodVector.add(target);
    				job.methodsInCurrentModel.add(methodVector);
    			}
    		}
    		
    		for ( String attribute : job.analyzeModel.getEditor().getAttributeNames(entity) ) {
    			// Attribute vector stores the owner entity in addition
    			// to the attribute entity itself. 
    			Vector<String> attributeVector = new Vector<String>();
    			attributeVector.add(entity);
    			attributeVector.add(attribute);
    			job.attributesInCurrentModel.add(attributeVector);
    		}
    		
    		for ( String parent : job.analyzeModel.getEditor().getParents(entity) ) {
    			// Parent vector stores both the parent and inherited entities' names.
    			Vector<String> parentVector = new Vector<String>();
    			parentVector.add(entity);
    			parentVector.add(parent);
    			job.parentsInCurrentModel.add(parentVector);
    		}
    	}
    }
    
    /**
     * Method for mapping the submodel for highlight request.
     * @param job Dot job to be handled.
     * @param drawNewElements Indicates wether or not the new elements are
     *   supposed to be drawn in the highlighted model. New elements are
     *   those that exist in the highlight request but do not exist in the
     *   current analyze model.
     */
    private void mapHighlightRequest(DotJob job, boolean drawNewElements) {
    	for ( String entity : job.entitiesRequestedForHighlighting ) {
    		for ( String method: job.highlightModel.getEditor().getMethodNames(entity) ) {
    			for ( String target: job.highlightModel.getEditor().
    					getEntitiesInfluencedByMethod(entity, method) ) {
    				// Method vector stores the method name plus its owner and
    				// target entities.
    				Vector<String> methodVector = new Vector<String>();
    				methodVector.add(entity);
    				methodVector.add(method);
    				methodVector.add(target);
    				job.methodsRequestedForHighlighting.add(methodVector);
    			}
    		}
    		
    		for ( String attribute : job.highlightModel.getEditor().getAttributeNames(entity) ) {
    			// Attribute vector stores the owner entity in addition
    			// to the attribute entity itself. 
    			Vector<String> attributeVector = new Vector<String>();
    			attributeVector.add(entity);
    			attributeVector.add(attribute);
    			job.attributesRequestedForHighlighting.add(attributeVector);
    		}
    		
    		for ( String parent : job.highlightModel.getEditor().getParents(entity) ) {
    			// Parent vector stores both the parent and inherited entities' names.
    			Vector<String> parentVector = new Vector<String>();
    			parentVector.add(entity);
    			parentVector.add(parent);
    			job.parentsRequestedForHighlighting.add(parentVector);
    		}
    		
    		if ( !job.entitiesInCurrentModel.contains(entity) && !drawNewElements ) continue;
			job.entitiesToBeHighlighted.add(entity);
    	}
    }
    
    /**
     * Method for highlighting submodels from the analyze model.
     * @param highlight The submodel to be highlighted. If this argument is
     *   null, then all applied highlights are removed.
     * @param drawNewElements Indicates wether or not those elements
     *   which do not exist in the current analyze model should be
     *   also drawn and highlighted.
     */
    public void highlight(AnalyzeModel highlight, boolean drawNewElements) {
    	DotJob job = new DotJob();
    	try {
    		job.analyzeModel.getEditor().merge(owner.getControlInterface().getAnalyzeModel());
    	} catch (AnalyzeModelException e) {
    		e.printStackTrace();
    	}
    	job.highlightModel = highlight;
    	
    	// Initialize sets for current entities and their relationships
    	// and then map those entities and their relationships which are
    	// requested to be highlighted.
    	job.entitiesInCurrentModel = owner.getControlInterface().getAnalyzeModel().getEditor().getEntityNames();
    	if ( highlight != null ) job.entitiesRequestedForHighlighting =
    		highlight.getEditor().getEntityNames();
    	
    	// If highlight is null, then just remove any highlightings that
    	// are currently applied.
    	if ( highlight != null ) {
        	
        	mapCurrentModel(job);
        	
        	mapHighlightRequest(job, drawNewElements);
        	
        	// Calculate methods to be highlighted.
	    	for ( Vector<String> method : job.methodsRequestedForHighlighting ) {
	    		if ( !job.methodsInCurrentModel.contains(method) &&
	    				!drawNewElements ) continue;
	    		job.methodsToBeHighlighted.add(method);
	    	}
	    	
	    	// Calculate attributes to be highlighted.
	    	for ( Vector<String> attribute : job.attributesRequestedForHighlighting ) {
	    		if ( !job.attributesInCurrentModel.contains(attribute) &&
	    				!drawNewElements ) continue;
	    		job.attributesToBeHighlighted.add(attribute);
	    	}
	    	
	    	// Calculate parents to be highlighted.
	    	for ( Vector<String> parent : job.parentsRequestedForHighlighting ) {
	    		if ( !job.parentsInCurrentModel.contains(parent) &&
	    				!drawNewElements ) continue;
	    		job.parentsToBeHighlighted.add(parent);
	    	}
    	}
    	
    	updateModel(job);
    	
    	// Destroy class sets associated with highlighting.
    }
    
    /**
     * Method for highlighting submodels from the analyze model.
     * As default we assume that new entities don't need to be drawn.
     * @param highlight Submodel to be highlighted.
     */
    public void highlight(AnalyzeModel highlight) {
    	highlight(highlight, false);
    }
    
    /**
     * Method for updating the model. This method pushes
     * the given dot job to the top of the dot job stack
     * and notifies the running thread about it. If no
     * updation thread is running yet, one is created.
     * @param job Dot job to be run next.
     */
    private void updateModel(final DotJob job) {
    	synchronized (DOT_JOB_STACK) {
    		DOT_JOB_STACK.push(job);
    	}
    	if ( EXECUTING_DOT_THREAD == null ) {
    		EXECUTING_DOT_THREAD = new Thread(new Runnable() {
				public void run() {
					executeDot();
				}        	
	        });
    		EXECUTING_DOT_THREAD.start();
    	} else {
    		synchronized (EXECUTING_DOT_THREAD) {
    			EXECUTING_DOT_THREAD.notify();
    		}
    	}
    }
    
    /**
     * Method for updating the analyze model.
     * @param analyzeModel New analyze model.
     */
    public void updateModel(AnalyzeModel analyzeModel) {
    	DotJob job = new DotJob();
    	try {
    		job.analyzeModel.getEditor().merge(analyzeModel);
    	} catch (AnalyzeModelException e) {
    		e.printStackTrace();
    	}
    	updateModel(job);
    }
    
    /**
     * This finalize method makes sure the thread
     * running for panel update shuts down.
     */
    protected void finalize() throws Throwable {
    	THREAD_RUNNING = false;
    	synchronized (EXECUTING_DOT_THREAD) {
    		if ( EXECUTING_DOT_THREAD != null ) {
    			EXECUTING_DOT_THREAD.notify();
    		}
    	}
    	synchronized (EXECUTING_EXPORT_THREAD) {
    		if ( EXECUTING_EXPORT_THREAD != null ) {
    			EXECUTING_EXPORT_THREAD.notify();
    		}
    	}
    	super.finalize();
    }
    
    /**
     * This method draws the graph generated by dot to the panel's
     * canvas.
     */
    public void paintComponent(Graphics g) {
    	super.setBackground(colorTheme.getBackgroundColorAsJavaObject());
        super.paintComponent(g);
        int x = 0, y = 0;
        // Update component size to match the scroller size.
        if (this.modelImage != null) {
            x = Math.max(0, (int) Math.floor((this.getWidth() - this.modelImage
                    .getWidth()) / 2));
            y = Math.max(0,
                    (int) Math.floor((this.getHeight() - this.modelImage
                            .getHeight()) / 2));

            g.drawImage(this.modelImage, x, y, this);
            
            this.setPreferredSize(new Dimension(this.modelImage.getWidth(), this.modelImage.getHeight()));
            this.revalidate();
        }
    }
    
    /**
     * This method should be executed in its own thread. This thread keeps
     * running until threadRunning class variable is set to false.
     * It takes the latest job from the dot job stack and disposes all the
     * other jobs in the stack at the same time. The newest job is then
     * modeled and drawn to the canvas. After that the thread sleeps until
     * waken again by notification [executing.notify()].
     */
	private void executeDot() {
		
		while ( THREAD_RUNNING ) {
			if ( !DOT_JOB_STACK.isEmpty() ) {
				
				long startTime = System.currentTimeMillis();
				DotJob job = null;
				
				synchronized (DOT_JOB_STACK) {
					job = DOT_JOB_STACK.pop();
					DOT_JOB_STACK.clear();
				}
				
				if ( job == null ) continue;
				
				File dotFile = createDotFile(job);
		        if (dotFile == null) {
		            System.err
		                    .println("Error in initializing dotFile, exiting updateAnalyzemodel.");
		            continue;
		        }
		
		        File imageFile = runDot(dotFile);
		        if (imageFile == null) {
		            System.err
		                    .println("Error in initializing imageFile, exiting updateAnalyzemodel.");
		            continue;
		        }
		        
		        // Check if image file exists... just to be sure.
		        if (!imageFile.exists()) {
		            System.err.println("DotPanel.java: imageFile (" + imageFile.getName() + ") not found.");
		            continue;
		        }
		
		        // If everything went ok, load image from file.
		        try {
		            this.modelImage = ImageIO.read(imageFile);
		        } catch (IOException e) {
		            System.err.println("DotPanel.java: Error in reading image from file.");
		            e.printStackTrace();
		            continue;
		        }
		
		        // Remove temp files
		        //dotFile.delete();
		        //imageFile.delete();
		        
		        this.repaint();
		        
		        // Uncomment the next line of code to confirm that the dot panel really
		        // is running in a separate thread.
		        // for ( int i = 0; i < 100000; System.out.println(++i) );
		        
		        timeElapsed = System.currentTimeMillis() - startTime;
		        if ( timerComponent != null ) {
		        	timerComponent.setText( " "+timeElapsed+" ms " );
		        }
			} else {
				try {
					synchronized (EXECUTING_DOT_THREAD) {
						EXECUTING_DOT_THREAD.wait();
					}
				} catch (Exception e) { e.printStackTrace(); }
			}
		}
	}
	
	public void setTimerComponent(JLabel timerComponent) {
		this.timerComponent = timerComponent;
	}	
}
